@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 # Copyright (c) 2012 Mikkel Schubert <MikkelSch@gmail.com>
 #
@@ -20,17 +20,10 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 #
-# Required due to use of NotImplementedError in setattr:
-# pylint: disable=R0921
 import re
 
-from paleomix.common.utilities import \
-     safe_coerce_to_tuple, \
-     Immutable, \
-     TotallyOrdered
-from paleomix.common.formats._graph import \
-     GraphError, \
-     _Graph
+from paleomix.common.utilities import safe_coerce_to_tuple, Immutable, TotallyOrdered
+from paleomix.common.formats._graph import GraphError, _Graph
 
 
 class NewickError(GraphError):
@@ -40,6 +33,7 @@ class NewickError(GraphError):
 class NewickParseError(NewickError):
     """Exception raised if errors occur during parsing
     of Newick strings."""
+
     pass
 
 
@@ -57,18 +51,16 @@ class Newick(TotallyOrdered, Immutable):
     into strings when the Newick string is generated. However, additional
     contraints apply when unrooting/rerooting trees (see below). """
 
-    def __init__(self, name = None, length = None, children = None):
+    def __init__(self, name=None, length=None, children=None):
         """See class documentation for constraints."""
 
         name = name or None
         length = length or None
         children = tuple(children or ())
         nw_hash = hash((name, length, children))
-        Immutable.__init__(self,
-                           name     = name,
-                           length   = length,
-                           children = children,
-                           _hash    = nw_hash)
+        Immutable.__init__(
+            self, name=name, length=length, children=children, _hash=nw_hash
+        )
 
         if not (self.children or self.name or self.length):
             raise NewickError("Leaf nodes MUST have either a name or a length")
@@ -84,13 +76,11 @@ class Newick(TotallyOrdered, Immutable):
             weight += 1
         object.__setattr__(self, "_weight", weight)
 
-
     @property
     def is_leaf(self):
         """Returns true if the node is a leaf (has no children)."""
         return not self.children
 
-
     def get_leaf_nodes(self):
         """Returns iterable for leaf-nodes accessible from this node."""
         if not self.is_leaf:
@@ -100,12 +90,10 @@ class Newick(TotallyOrdered, Immutable):
         else:
             yield self
 
-
     def get_leaf_names(self):
         for node in self.get_leaf_nodes():
             yield node.name
 
-
     def reroot_on_taxa(self, taxa):
         """Returns the Newick tree from this node, but rooted on the midpoint
         of the branch leading to one or more taxa. Note that the taxa are not
@@ -114,7 +102,6 @@ class Newick(TotallyOrdered, Immutable):
         function."""
         return _NewickGraph(self).reroot_on_taxa(taxa)
 
-
     def reroot_on_midpoint(self):
         """Returns the newick tree from this node, but rooted on the midpoint
         of the tree. That is to say that a root node is added at the exact
@@ -129,12 +116,11 @@ class Newick(TotallyOrdered, Immutable):
         are allowed), but note that rerooting behavior around nodes with
         length zero may yield unexpected results."""
         if len(list(self.get_leaf_nodes())) < 2:
-            return self # No meaningful way to reroot such trees
+            return self  # No meaningful way to reroot such trees
 
         return _NewickGraph(self).reroot_on_midpoint()
 
-
-    def add_support(self, bootstraps, fmt = "{Support}"):
+    def add_support(self, bootstraps, fmt="{Support}"):
         """Adds support values to the current tree, based on a set of trees containing
         the same taxa. It is assumed that the support trees represent unrooted or
         arbitarily rooted trees, and no weight is given to the rooted topology of these
@@ -155,21 +141,24 @@ class Newick(TotallyOrdered, Immutable):
         leaf_names_lst = list(self.get_leaf_names())
         leaf_names = frozenset(leaf_names_lst)
         if len(leaf_names) != len(leaf_names_lst):
-            raise NewickError("Cannot add support values to trees with duplicate leaf names")
+            raise NewickError(
+                "Cannot add support values to trees with duplicate leaf names"
+            )
 
-        bootstraps   = safe_coerce_to_tuple(bootstraps)
+        bootstraps = safe_coerce_to_tuple(bootstraps)
         for support_tree in bootstraps:
             support_tree_names = frozenset(support_tree.get_leaf_names())
             if leaf_names != support_tree_names:
-                raise NewickError("Support tree does not contain same set of leaf nodes")
+                raise NewickError(
+                    "Support tree does not contain same set of leaf nodes"
+                )
 
-            support_graph      = _NewickGraph(support_tree)
+            support_graph = _NewickGraph(support_tree)
             for clade in support_graph.get_clade_names():
                 clade_counts[clade] = clade_counts.get(clade, 0) + 1
 
         return self._add_support(self, len(bootstraps), clade_counts, fmt)
 
-
     @classmethod
     def from_string(cls, string):
         """Parses a Newick string and returns a representation of the tree.
@@ -189,34 +178,33 @@ class Newick(TotallyOrdered, Immutable):
 
         return top_node
 
-
     def __lt__(self, other):
         """See TotallyOrdered"""
         if not isinstance(other, Newick):
             return NotImplemented
 
-        # pylint: disable=W0212
-        return (-self._weight, self.name, self.length, self.children) \
-          < (-other._weight, other.name, other.length, other.children)
-
+        return (-self._weight, self.name, self.length, self.children) < (
+            -other._weight,
+            other.name,
+            other.length,
+            other.children,
+        )
 
     def __hash__(self):
         """Hashing function, see 'hash'."""
         return self._hash
 
-
     def __repr__(self):
         """Representation corresponds to the Newick string for the (sub)tree,
         which can be parsed by 'from_string'."""
         return "%s;" % (self._to_str(),)
 
-
     def _to_str(self):
         fields = []
         if self.children:
             fields.append("(")
             for child in self.children:
-                fields.append(child._to_str()) # pylint: disable=W0212
+                fields.append(child._to_str())
                 fields.append(",")
             fields.pop()
             fields.append(")")
@@ -227,7 +215,6 @@ class Newick(TotallyOrdered, Immutable):
             fields.append(str(self.length))
         return "".join(fields)
 
-
     def _add_support(self, node, total, clade_counts, fmt):
         """Recursively annotates a subtree with support values,
         excepting leaf nodes (where the name is preserved) and
@@ -235,27 +222,28 @@ class Newick(TotallyOrdered, Immutable):
         if node.is_leaf:
             return node
 
-        clade   = frozenset(leaf.name for leaf in node.get_leaf_nodes())
+        clade = frozenset(leaf.name for leaf in node.get_leaf_nodes())
         support = clade_counts.get(clade, 0)
-        name = fmt.format(Support    = support,
-                          Percentage = (support * 100.0) / (total or 1),
-                          Fraction   = (support * 1.0) / (total or 1))
+        name = fmt.format(
+            Support=support,
+            Percentage=(support * 100.0) / (total or 1),
+            Fraction=(support * 1.0) / (total or 1),
+        )
 
         children = []
         for child in node.children:
             children.append(self._add_support(child, total, clade_counts, fmt))
 
-        return Newick(name     = (None if (node is self) else name),
-                      length   = node.length,
-                      children = children)
-
-
-
+        return Newick(
+            name=(None if (node is self) else name),
+            length=node.length,
+            children=children,
+        )
 
 
 ################################################################################
 ################################################################################
-## Functions related to NEWICK parsing
+# Functions related to NEWICK parsing
 
 _TOKENIZER = re.compile("([():,;])")
 _NODE_KEYS = frozenset(("name", "length", "children"))
@@ -283,21 +271,25 @@ def _parse_tokens(tokens):
     children.append(child)
 
     if any(child is None for child in children):
-        raise NewickParseError("Implicit leaf nodes (no name OR length) are not allowed")
+        raise NewickParseError(
+            "Implicit leaf nodes (no name OR length) are not allowed"
+        )
     elif not tokens or (tokens[0] != ")"):
-        raise NewickParseError("Malformed Newick string, contains unbalanced parantheses")
+        raise NewickParseError(
+            "Malformed Newick string, contains unbalanced parantheses"
+        )
     tokens.pop(0)
 
-    return _parse_child(tokens, children = children)
+    return _parse_child(tokens, children=children)
 
 
-def _parse_child(tokens, children = None):
+def _parse_child(tokens, children=None):
     if tokens and tokens[0] == "(":
         return _parse_tokens(tokens)
 
     name, length = None, None
     while tokens and (tokens[0] not in ",);"):
-        if (tokens[0] == ":"):
+        if tokens[0] == ":":
             if length is not None:
                 raise NewickParseError("Node has multiple length values")
             tokens.pop(0)
@@ -310,15 +302,13 @@ def _parse_child(tokens, children = None):
     if not (name or length or children):
         raise NewickParseError("Parsing of implicit nodes not supported")
 
-    return Newick(name     = name,
-                  length   = length,
-                  children = children)
-
+    return Newick(name=name, length=length, children=children)
 
 
 ################################################################################
 ################################################################################
-## Class related to tree manipulations
+# Class related to tree manipulations
+
 
 class _NewickGraph(_Graph):
     def __init__(self, node):
@@ -326,17 +316,15 @@ class _NewickGraph(_Graph):
         self._collect_names_and_blengths(node)
         self.prune_uninformative_nodes()
 
-
     def _collect_names_and_blengths(self, c_node):
         c_node_id = id(c_node)
 
-        self.set_name(c_node_id,  c_node.name)
+        self.set_name(c_node_id, c_node.name)
         for child in c_node.children:
             child_id = id(child)
             self.add_connection(c_node_id, child_id, child.length)
             self._collect_names_and_blengths(child)
 
-
     def rebuild_tree(self, parent_id, node_id):
         """Rebuilds a newick tree starting at a node with id
         'node_id' and a parent with id 'parent_id' (or the
@@ -352,6 +340,4 @@ class _NewickGraph(_Graph):
         if isinstance(blength, float):
             blength = repr(blength)
 
-        return Newick(name     = self.names.get(node_id),
-                      length   = blength,
-                      children = children)
+        return Newick(name=self.names.get(node_id), length=blength, children=children)
